@@ -1,7 +1,7 @@
 // LNP/BP client-side-validation foundation libraries implementing LNPBP
 // specifications & standards (LNPBP-4, 7, 8, 9, 42, 81)
 //
-// Written in 2019-2021 by
+// Written in 2019-2022 by
 //     Dr. Maxim Orlovsky <orlovsky@pandoracore.com>
 //
 // To the extent possible under law, the author(s) have dedicated all
@@ -13,16 +13,25 @@
 // software. If not, see <https://opensource.org/licenses/Apache-2.0>.
 
 use std::io;
+use std::io::{Read, Write};
 
-use bitcoin::bech32::u5;
-use bitcoin::util::address::{self, Address};
+use bitcoin::psbt::{self, PsbtSighashType, TapTree};
+use bitcoin::secp256k1::{ecdsa, schnorr, Secp256k1};
+use bitcoin::util::address::{self, Address, WitnessVersion};
 use bitcoin::util::bip32;
-use bitcoin::util::psbt::PartiallySignedTransaction;
+use bitcoin::util::taproot::{
+    ControlBlock, FutureLeafVersion, LeafVersion, ScriptLeaf, TapBranchHash,
+    TapLeafHash, TapSighashHash, TapTweakHash, TaprootBuilder,
+    TaprootMerkleBranch,
+};
 use bitcoin::{
-    secp256k1, Amount, BlockHash, OutPoint, PubkeyHash, Script, ScriptHash,
-    SigHash, Transaction, TxIn, TxOut, Txid, WPubkeyHash, WScriptHash, Wtxid,
+    schnorr as bip340, secp256k1, Amount, BlockHash, EcdsaSig,
+    EcdsaSighashType, KeyPair, OutPoint, PubkeyHash, SchnorrSig,
+    SchnorrSighashType, Script, ScriptHash, Sighash, Transaction, TxIn, TxOut,
+    Txid, WPubkeyHash, WScriptHash, Witness, Wtxid, XOnlyPublicKey,
     XpubIdentifier,
 };
+use bitcoin_hashes::sha256;
 
 use crate::{strategies, Error, Strategy, StrictDecode, StrictEncode};
 
@@ -50,9 +59,75 @@ impl Strategy for ScriptHash {
 impl Strategy for WScriptHash {
     type Strategy = strategies::HashFixedBytes;
 }
-impl Strategy for SigHash {
+impl Strategy for Sighash {
     type Strategy = strategies::HashFixedBytes;
 }
+impl Strategy for TapBranchHash {
+    type Strategy = strategies::HashFixedBytes;
+}
+impl Strategy for TapLeafHash {
+    type Strategy = strategies::HashFixedBytes;
+}
+impl Strategy for TapTweakHash {
+    type Strategy = strategies::HashFixedBytes;
+}
+impl Strategy for TapSighashHash {
+    type Strategy = strategies::HashFixedBytes;
+}
+
+impl StrictEncode for LeafVersion {
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_consensus().strict_encode(e)
+    }
+}
+
+impl StrictDecode for LeafVersion {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        let leaf_version = u8::strict_decode(d)?;
+        LeafVersion::from_consensus(leaf_version).map_err(|_| {
+            Error::DataIntegrityError(format!(
+                "incorrect LeafVersion `{}`",
+                leaf_version
+            ))
+        })
+    }
+}
+
+impl StrictEncode for FutureLeafVersion {
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_consensus().strict_encode(e)
+    }
+}
+
+impl StrictDecode for FutureLeafVersion {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        match LeafVersion::strict_decode(d)? {
+            LeafVersion::TapScript => {
+                Err(Error::DataIntegrityError(s!("known LeafVersion was \
+                                                  found while decoding \
+                                                  FutureLeafVersion")))
+            }
+            LeafVersion::Future(version) => Ok(version),
+        }
+    }
+}
+
+impl StrictEncode for TaprootMerkleBranch {
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl StrictDecode for TaprootMerkleBranch {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        let data = Vec::<sha256::Hash>::strict_decode(d)?;
+        TaprootMerkleBranch::from_inner(data).map_err(|_| {
+            Error::DataIntegrityError(s!(
+                "taproot merkle branch length exceeds 128 consensus limit"
+            ))
+        })
+    }
+}
 
 impl StrictEncode for secp256k1::SecretKey {
     #[inline]
@@ -72,6 +147,48 @@ impl StrictDecode for secp256k1::SecretKey {
     }
 }
 
+impl StrictEncode for bip340::TweakedKeyPair {
+    #[inline]
+    fn strict_encode<E: io::Write>(&self, e: E) -> Result<usize, Error> {
+        self.into_inner().strict_encode(e)
+    }
+}
+
+impl StrictDecode for bip340::TweakedKeyPair {
+    #[inline]
+    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
+        let mut buf = [0u8; secp256k1::constants::SECRET_KEY_SIZE];
+        d.read_exact(&mut buf)?;
+        let secp = Secp256k1::signing_only();
+        Ok(Self::dangerous_assume_tweaked(
+            KeyPair::from_seckey_slice(&secp, &buf).map_err(|_| {
+                Error::DataIntegrityError(
+                    "invalid BIP340 keypair data".to_string(),
+                )
+            })?,
+        ))
+    }
+}
+
+impl StrictEncode for KeyPair {
+    #[inline]
+    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(e.write(&self.secret_bytes())?)
+    }
+}
+
+impl StrictDecode for KeyPair {
+    #[inline]
+    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
+        let mut buf = [0u8; secp256k1::constants::SECRET_KEY_SIZE];
+        d.read_exact(&mut buf)?;
+        let secp = Secp256k1::signing_only();
+        Self::from_seckey_slice(&secp, &buf).map_err(|_| {
+            Error::DataIntegrityError("invalid BIP340 keypair data".to_string())
+        })
+    }
+}
+
 impl StrictEncode for secp256k1::PublicKey {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
@@ -99,44 +216,52 @@ impl StrictDecode for secp256k1::PublicKey {
     }
 }
 
-impl StrictEncode for secp256k1::schnorrsig::PublicKey {
+impl StrictEncode for XOnlyPublicKey {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        Ok(e.write(&[0x02_u8])? + e.write(&self.serialize())?)
+        Ok(e.write(&self.serialize())?)
     }
 }
 
-impl StrictDecode for secp256k1::schnorrsig::PublicKey {
+impl StrictDecode for XOnlyPublicKey {
     #[inline]
     fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut buf =
-            [0u8; secp256k1::constants::SCHNORRSIG_PUBLIC_KEY_SIZE + 1];
+        let mut buf = [0u8; secp256k1::constants::SCHNORR_PUBLIC_KEY_SIZE];
         d.read_exact(&mut buf)?;
-        if buf[0] != 0x02 {
-            return Err(Error::DataIntegrityError(s!("invalid public key \
-                                                     data: BIP340 keys \
-                                                     must be serialized \
-                                                     with `0x02` prefix \
-                                                     byte")));
-        }
-        Self::from_slice(&buf[1..]).map_err(|_| {
+        Self::from_slice(&buf[..]).map_err(|_| {
             Error::DataIntegrityError(s!("invalid public key data"))
         })
     }
 }
 
-// TODO: #17 Implement strict encoding for `KeyPair` type once there will be a
-//       way to serialize its inner data in Secpk256k1 lib (see
-//       <https://github.com/rust-bitcoin/rust-secp256k1/issues/298>)
+impl StrictEncode for bip340::TweakedPublicKey {
+    #[inline]
+    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(e.write(&self.serialize())?)
+    }
+}
 
-impl StrictEncode for secp256k1::Signature {
+impl StrictDecode for bip340::TweakedPublicKey {
+    #[inline]
+    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
+        let mut buf = [0u8; secp256k1::constants::SCHNORR_PUBLIC_KEY_SIZE];
+        d.read_exact(&mut buf)?;
+        Ok(Self::dangerous_assume_tweaked(
+            XOnlyPublicKey::from_slice(&buf[..]).map_err(|_| {
+                Error::DataIntegrityError(s!("invalid public key data"))
+            })?,
+        ))
+    }
+}
+
+impl StrictEncode for ecdsa::Signature {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
         Ok(e.write(&self.serialize_compact())?)
     }
 }
 
-impl StrictDecode for secp256k1::Signature {
+impl StrictDecode for ecdsa::Signature {
     #[inline]
     fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
         let mut buf = [0u8; secp256k1::constants::COMPACT_SIGNATURE_SIZE];
@@ -149,17 +274,17 @@ impl StrictDecode for secp256k1::Signature {
     }
 }
 
-impl StrictEncode for secp256k1::schnorrsig::Signature {
+impl StrictEncode for schnorr::Signature {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
         Ok(e.write(&self[..])?)
     }
 }
 
-impl StrictDecode for secp256k1::schnorrsig::Signature {
+impl StrictDecode for schnorr::Signature {
     #[inline]
     fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut buf = [0u8; secp256k1::constants::SCHNORRSIG_SIGNATURE_SIZE];
+        let mut buf = [0u8; secp256k1::constants::SCHNORR_SIGNATURE_SIZE];
         d.read_exact(&mut buf)?;
         Self::from_slice(&buf).map_err(|_| {
             Error::DataIntegrityError(
@@ -169,6 +294,82 @@ impl StrictDecode for secp256k1::schnorrsig::Signature {
     }
 }
 
+impl StrictEncode for PsbtSighashType {
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_u32().strict_encode(e)
+    }
+}
+
+impl StrictDecode for PsbtSighashType {
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        u32::strict_decode(d).map(PsbtSighashType::from_u32)
+    }
+}
+
+impl StrictEncode for EcdsaSighashType {
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_u32().strict_encode(e)
+    }
+}
+
+impl StrictDecode for EcdsaSighashType {
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Ok(EcdsaSighashType::from_consensus(u32::strict_decode(d)?))
+    }
+}
+
+impl StrictEncode for SchnorrSighashType {
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        (*self as u8).strict_encode(e)
+    }
+}
+
+impl StrictDecode for SchnorrSighashType {
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        SchnorrSighashType::from_u8(u8::strict_decode(d)?).map_err(|_| {
+            Error::DataIntegrityError(s!("invalid BIP431 SighashType value"))
+        })
+    }
+}
+
+impl StrictEncode for EcdsaSig {
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_vec().strict_encode(e)
+    }
+}
+
+impl StrictDecode for EcdsaSig {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        EcdsaSig::from_slice(&Vec::strict_decode(d)?).map_err(|_| {
+            Error::DataIntegrityError(s!(
+                "invalid ECDSA tx input signature data"
+            ))
+        })
+    }
+}
+
+impl StrictEncode for SchnorrSig {
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.to_vec().strict_encode(e)
+    }
+}
+
+impl StrictDecode for SchnorrSig {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        SchnorrSig::from_slice(&Vec::strict_decode(d)?).map_err(|_| {
+            Error::DataIntegrityError(s!("invalid BIP431 signature data"))
+        })
+    }
+}
+
 #[doc(hidden)]
 #[allow(useless_deprecated)]
 #[deprecated(
@@ -180,9 +381,9 @@ impl StrictEncode for bitcoin::PublicKey {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
         Ok(if self.compressed {
-            e.write(&self.key.serialize())?
+            e.write(&self.inner.serialize())?
         } else {
-            e.write(&self.key.serialize_uncompressed())?
+            e.write(&self.inner.serialize_uncompressed())?
         })
     }
 }
@@ -232,6 +433,9 @@ impl StrictDecode for bitcoin::PublicKey {
 impl Strategy for OutPoint {
     type Strategy = strategies::BitcoinConsensus;
 }
+impl Strategy for Witness {
+    type Strategy = strategies::BitcoinConsensus;
+}
 impl Strategy for TxOut {
     type Strategy = strategies::BitcoinConsensus;
 }
@@ -241,9 +445,6 @@ impl Strategy for TxIn {
 impl Strategy for Transaction {
     type Strategy = strategies::BitcoinConsensus;
 }
-impl Strategy for PartiallySignedTransaction {
-    type Strategy = strategies::BitcoinConsensus;
-}
 
 impl StrictEncode for address::Payload {
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
@@ -255,7 +456,7 @@ impl StrictEncode for address::Payload {
                 33u8.strict_encode(&mut e)? + sh.strict_encode(&mut e)?
             }
             address::Payload::WitnessProgram { version, program } => {
-                version.to_u8().strict_encode(&mut e)?
+                version.into_num().strict_encode(&mut e)?
                     + program.strict_encode(&mut e)?
             }
         })
@@ -271,9 +472,8 @@ impl StrictDecode for address::Payload {
             33u8 => {
                 address::Payload::ScriptHash(ScriptHash::strict_decode(&mut d)?)
             }
-            // TODO: #18 Update to `WitnessVersion` upon bitcoin 0.26.1 release
             version if version <= 16 => address::Payload::WitnessProgram {
-                version: u5::try_from_u8(version)
+                version: WitnessVersion::from_num(version)
                     .expect("bech32::u8 decider is broken"),
                 program: StrictDecode::strict_decode(&mut d)?,
             },
@@ -326,6 +526,34 @@ impl StrictDecode for Script {
     }
 }
 
+impl StrictEncode for ControlBlock {
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(self.size().strict_encode(&mut e)? + self.encode(e)?)
+    }
+}
+
+impl StrictDecode for ControlBlock {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        let data = Vec::<u8>::strict_decode(d)?;
+        Self::from_slice(&data)
+            .map_err(|err| Error::DataIntegrityError(err.to_string()))
+    }
+}
+
+impl StrictEncode for ScriptLeaf {
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.script(), self.leaf_version()))
+    }
+}
+
+/* TODO: Uncomment once ScriptLeaf::new will become public
+impl StrictDecode for ScriptLeaf {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Ok(ScriptLeaf::new())
+    }
+}
+ */
+
 impl StrictEncode for bitcoin::Network {
     #[inline]
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
@@ -459,6 +687,69 @@ impl StrictDecode for bip32::ExtendedPrivKey {
     }
 }
 
+impl StrictEncode for psbt::raw::Key {
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.type_value, self.key))
+    }
+}
+
+impl StrictDecode for psbt::raw::Key {
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        Ok(strict_decode_self!(d; type_value, key; crate))
+    }
+}
+
+impl StrictEncode for psbt::raw::Pair {
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.key, self.value))
+    }
+}
+
+impl StrictDecode for psbt::raw::Pair {
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        Ok(strict_decode_self!(d; key, value; crate))
+    }
+}
+
+impl StrictEncode for psbt::raw::ProprietaryKey {
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.prefix, self.subtype, self.key))
+    }
+}
+
+impl StrictDecode for psbt::raw::ProprietaryKey {
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        Ok(strict_decode_self!(d; prefix, subtype, key; crate))
+    }
+}
+
+impl StrictEncode for TapTree {
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        impl StrictEncode for &Script {
+            fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+                Script::strict_encode(self, e)
+            }
+        }
+        self.script_leaves()
+            .cloned()
+            .collect::<Vec<_>>()
+            .strict_encode(e)
+    }
+}
+
+impl StrictDecode for TapTree {
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        let builder = Vec::<(u8, Script)>::strict_decode(d)?
+            .into_iter()
+            .try_fold(TaprootBuilder::new(), |builder, (depth, script)| {
+                builder.add_leaf(depth, script)
+            })
+            .map_err(|err| Error::DataIntegrityError(err.to_string()))?;
+        TapTree::from_builder(builder)
+            .map_err(|_| Error::DataIntegrityError(s!("incomplete tree")))
+    }
+}
+
 #[cfg(test)]
 pub(crate) mod test {
     use std::str::FromStr;
@@ -468,9 +759,9 @@ pub(crate) mod test {
     use bitcoin::hashes::Hash;
     use bitcoin::secp256k1::Message;
     use bitcoin_hashes::{hash160, hmac, ripemd160, sha256, sha256d, sha256t};
+    use strict_encoding_test::*;
 
     use super::*;
-    use crate::test_helpers::*;
 
     #[test]
     fn test_encoding_hashes() {
@@ -552,7 +843,7 @@ pub(crate) mod test {
         )
         .unwrap();
         test_encoding_roundtrip(
-            &SigHash::from_slice(&HASH256_BYTES).unwrap(),
+            &Sighash::from_slice(&HASH256_BYTES).unwrap(),
             HASH256_BYTES,
         )
         .unwrap();
@@ -589,8 +880,7 @@ pub(crate) mod test {
         ];
         let sk = secp256k1::SecretKey::from_slice(&SK_BYTES).unwrap();
         let _sk_bip340 =
-            secp256k1::schnorrsig::KeyPair::from_seckey_slice(&secp, &SK_BYTES)
-                .unwrap();
+            secp256k1::KeyPair::from_seckey_slice(&secp, &SK_BYTES).unwrap();
         // TODO: #17 implement KeyPair serialization testing
         test_encoding_roundtrip(&sk, &SK_BYTES[..])
     }
@@ -615,18 +905,17 @@ pub(crate) mod test {
             0x19, 0xeb, 0xfa, 0x57, 0xda, 0x7c, 0xff, 0x3a, 0xff, 0x6e, 0x81,
             0x9e, 0x4e, 0xe9, 0x71, 0xd8, 0x6b, 0x5e, 0x61, 0x87, 0x5d,
         ];
-        static PK_BYTES_ONEKEY: [u8; 33] = [
-            0x02, 0x79, 0xbe, 0x66, 0x7e, 0xf9, 0xdc, 0xbb, 0xac, 0x55, 0xa0,
-            0x62, 0x95, 0xce, 0x87, 0x0b, 0x07, 0x02, 0x9b, 0xfc, 0xdb, 0x2d,
-            0xce, 0x28, 0xd9, 0x59, 0xf2, 0x81, 0x5b, 0x16, 0xf8, 0x17, 0x98,
+        static PK_BYTES_ONEKEY: [u8; 32] = [
+            0x79, 0xbe, 0x66, 0x7e, 0xf9, 0xdc, 0xbb, 0xac, 0x55, 0xa0, 0x62,
+            0x95, 0xce, 0x87, 0x0b, 0x07, 0x02, 0x9b, 0xfc, 0xdb, 0x2d, 0xce,
+            0x28, 0xd9, 0x59, 0xf2, 0x81, 0x5b, 0x16, 0xf8, 0x17, 0x98,
         ];
 
         let secp_pk_02 =
             secp256k1::PublicKey::from_slice(&PK_BYTES_02).unwrap();
         let secp_pk_03 =
             secp256k1::PublicKey::from_slice(&PK_BYTES_03).unwrap();
-        let secp_pk_one =
-            secp256k1::PublicKey::from_slice(&PK_BYTES_ONEKEY).unwrap();
+        let secp_pk_one = XOnlyPublicKey::from_slice(&PK_BYTES_ONEKEY).unwrap();
         test_encoding_roundtrip(&secp_pk_02, PK_BYTES_02).unwrap();
         test_encoding_roundtrip(&secp_pk_03, PK_BYTES_03).unwrap();
         test_encoding_roundtrip(&secp_pk_one, PK_BYTES_ONEKEY).unwrap();
@@ -641,52 +930,35 @@ pub(crate) mod test {
 
         let sk_one = secp256k1::PublicKey::from_secret_key(
             &secp256k1::Secp256k1::new(),
-            &secp256k1::key::ONE_KEY,
+            &secp256k1::ONE_KEY,
         );
 
         let pubkey_02 = bitcoin::PublicKey::from_slice(&PK_BYTES_02).unwrap();
         let pubkey_03 = bitcoin::PublicKey::from_slice(&PK_BYTES_03).unwrap();
         let pubkey_04 = bitcoin::PublicKey::from_slice(&PK_BYTES_04).unwrap();
-        let one_key = bitcoin::PublicKey {
-            compressed: true,
-            key: sk_one,
-        };
+        let one_key = bitcoin::PublicKey::new(sk_one);
         test_encoding_roundtrip(&pubkey_02, PK_BYTES_02).unwrap();
         test_encoding_roundtrip(&pubkey_03, PK_BYTES_03).unwrap();
         test_encoding_roundtrip(&pubkey_04, PK_BYTES_04).unwrap();
-        test_encoding_roundtrip(&one_key, PK_BYTES_ONEKEY).unwrap();
-        assert_eq!(secp_pk_02, pubkey_02.key);
-        assert_eq!(secp_pk_02, pubkey_02.key);
-        assert_eq!(secp_pk_02, pubkey_02.key);
-        assert_eq!(secp_pk_03, pubkey_03.key);
-        assert_eq!(secp_pk_one, one_key.key);
-        assert_eq!(pubkey_03.key, pubkey_04.key);
+        assert_eq!(secp_pk_02, pubkey_02.inner);
+        assert_eq!(secp_pk_02, pubkey_02.inner);
+        assert_eq!(secp_pk_02, pubkey_02.inner);
+        assert_eq!(secp_pk_03, pubkey_03.inner);
+        assert_ne!(
+            &secp_pk_one.serialize()[..],
+            &one_key.inner.serialize()[..]
+        );
+        assert_eq!(pubkey_03.inner, pubkey_04.inner);
 
         let xcoordonly_02 =
-            secp256k1::schnorrsig::PublicKey::from_slice(&PK_BYTES_02[1..])
-                .unwrap();
+            XOnlyPublicKey::from_slice(&PK_BYTES_02[1..]).unwrap();
         let xcoordonly_one =
-            secp256k1::schnorrsig::PublicKey::from_slice(&PK_BYTES_ONEKEY[1..])
-                .unwrap();
-        test_encoding_roundtrip(&xcoordonly_02, PK_BYTES_02).unwrap();
+            XOnlyPublicKey::from_slice(&PK_BYTES_ONEKEY[..]).unwrap();
+        test_encoding_roundtrip(&xcoordonly_02, &PK_BYTES_02[1..]).unwrap();
         test_encoding_roundtrip(&xcoordonly_one, PK_BYTES_ONEKEY).unwrap();
-        assert_eq!(
-            secp256k1::schnorrsig::PublicKey::strict_decode(&PK_BYTES_03[..]),
-            Err(Error::DataIntegrityError(s!("invalid public key data: \
-                                              BIP340 keys must be \
-                                              serialized with `0x02` \
-                                              prefix byte")))
-        );
-        assert_eq!(
-            secp256k1::schnorrsig::PublicKey::strict_decode(&PK_BYTES_04[..]),
-            Err(Error::DataIntegrityError(s!("invalid public key data: \
-                                              BIP340 keys must be \
-                                              serialized with `0x02` \
-                                              prefix byte")))
-        );
         assert_eq!(xcoordonly_02.serialize(), secp_pk_02.serialize()[1..]);
         assert_eq!(xcoordonly_02.serialize(), secp_pk_03.serialize()[1..]);
-        assert_eq!(xcoordonly_one.serialize(), secp_pk_one.serialize()[1..]);
+        assert_eq!(xcoordonly_one.serialize(), one_key.inner.serialize()[1..]);
     }
 
     #[test]
@@ -724,12 +996,12 @@ pub(crate) mod test {
     ];
 
     static SCHNORR_BYTES: [u8; 64] = [
-        0x16, 0x2A, 0xDD, 0x99, 0x90, 0x6A, 0x93, 0x04, 0xD4, 0xD0, 0x79, 0x02,
-        0x73, 0xAA, 0xD1, 0xB3, 0x54, 0x13, 0x6C, 0x32, 0xBE, 0x53, 0xF0, 0x7D,
-        0xF8, 0x0F, 0x74, 0xB3, 0xF7, 0xBF, 0x99, 0xAE, 0x91, 0x13, 0x45, 0x76,
-        0xD0, 0x4B, 0x8A, 0x2D, 0x82, 0xDA, 0x7F, 0xE9, 0xDA, 0x43, 0x99, 0xF3,
-        0xBD, 0x99, 0xDF, 0x6B, 0xA3, 0xC6, 0xF9, 0xB4, 0xFB, 0x5E, 0x62, 0xDB,
-        0x1B, 0x15, 0xCF, 0xF3,
+        0xb2, 0xe6, 0x87, 0x0a, 0x24, 0xa5, 0xaf, 0x30, 0x54, 0xbd, 0xd0, 0x25,
+        0x23, 0xb4, 0xbd, 0xc0, 0x0a, 0xce, 0x93, 0xa7, 0xa8, 0xa4, 0x95, 0xb0,
+        0x92, 0x41, 0x87, 0xca, 0x61, 0x90, 0x71, 0x8e, 0xab, 0xe3, 0x8c, 0x3d,
+        0x12, 0x55, 0xad, 0x42, 0x9d, 0xff, 0x36, 0x93, 0x66, 0x2c, 0x3f, 0x0b,
+        0x5a, 0x57, 0xdd, 0x70, 0x8d, 0x53, 0xb0, 0xc3, 0x37, 0x3f, 0xec, 0xfb,
+        0xad, 0x82, 0x34, 0xa0,
     ];
 
     #[test]
@@ -744,35 +1016,34 @@ pub(crate) mod test {
 
         let sk_ecdsa = secp256k1::SecretKey::from_slice(&KEY).unwrap();
         let sk_schnorr =
-            secp256k1::schnorrsig::KeyPair::from_seckey_slice(&secp, &KEY)
-                .unwrap();
+            secp256k1::KeyPair::from_seckey_slice(&secp, &KEY).unwrap();
 
         let pk_ecdsa = secp256k1::PublicKey::from_secret_key(&secp, &sk_ecdsa);
-        let pk_schnorr =
-            secp256k1::schnorrsig::PublicKey::from_keypair(&secp, &sk_schnorr);
+        let pk_schnorr = XOnlyPublicKey::from_keypair(&sk_schnorr);
         let msg = Message::from_slice(&[1u8; 32]).unwrap();
 
-        let ecdsa = secp.sign(&msg, &sk_ecdsa);
+        let ecdsa = secp.sign_ecdsa(&msg, &sk_ecdsa);
         test_encoding_roundtrip(&ecdsa, &ECDSA_BYTES).unwrap();
-        assert!(secp.verify(&msg, &ecdsa, &pk_ecdsa).is_ok());
+        assert!(secp.verify_ecdsa(&msg, &ecdsa, &pk_ecdsa).is_ok());
 
-        let schnorr = secp.schnorrsig_sign_no_aux_rand(&msg, &sk_schnorr);
+        let schnorr =
+            secp.sign_schnorr_with_aux_rand(&msg, &sk_schnorr, &[0u8; 32]);
         test_encoding_roundtrip(&schnorr, &SCHNORR_BYTES).unwrap();
-        assert!(secp.schnorrsig_verify(&schnorr, &msg, &pk_schnorr).is_ok());
+        assert!(secp.verify_schnorr(&schnorr, &msg, &pk_schnorr).is_ok());
 
         // Schnorr signature can be deserialized as ECDSA and vice verse,
         // (since there is no encoding-level way of verifying its type)
         // but MUST be invalid upon signature validation
-        let schnorr_as_ecdsa: secp256k1::Signature =
+        let schnorr_as_ecdsa: ecdsa::Signature =
             test_vec_decoding_roundtrip(&SCHNORR_BYTES).unwrap();
-        let ecdsa_as_schnorr: secp256k1::schnorrsig::Signature =
+        let ecdsa_as_schnorr: schnorr::Signature =
             test_vec_decoding_roundtrip(&ECDSA_BYTES).unwrap();
         assert_eq!(
-            secp.verify(&msg, &schnorr_as_ecdsa, &pk_ecdsa),
+            secp.verify_ecdsa(&msg, &schnorr_as_ecdsa, &pk_ecdsa),
             Err(secp256k1::Error::IncorrectSignature)
         );
         assert_eq!(
-            secp.schnorrsig_verify(&ecdsa_as_schnorr, &msg, &pk_schnorr),
+            secp.verify_schnorr(&ecdsa_as_schnorr, &msg, &pk_schnorr),
             Err(secp256k1::Error::InvalidSignature)
         );
     }
@@ -780,14 +1051,13 @@ pub(crate) mod test {
     #[test]
     #[should_panic(expected = "UnexpectedEof")]
     fn test_garbagedata_ecdsa() {
-        secp256k1::Signature::strict_decode(&ECDSA_BYTES[5..]).unwrap();
+        ecdsa::Signature::strict_decode(&ECDSA_BYTES[5..]).unwrap();
     }
 
     #[test]
     #[should_panic(expected = "UnexpectedEof")]
     fn test_garbagedata_schnorrsig() {
-        secp256k1::schnorrsig::Signature::strict_decode(&SCHNORR_BYTES[5..])
-            .unwrap();
+        schnorr::Signature::strict_decode(&SCHNORR_BYTES[5..]).unwrap();
     }
 
     #[test]
@@ -1044,34 +1314,6 @@ pub(crate) mod test {
         test_encoding_roundtrip(&txout_legacy, &txout_legacy_bytes).unwrap();
     }
 
-    #[test]
-    fn test_psbt() {
-        let psbt_bytes = Vec::from_hex(
-            "70736274ff0100750200000001268171371edff285e937adeea4b37b78000c0566\
-            cbb3ad64641713ca42171bf60000000000feffffff02d3dff505000000001976a91\
-            4d0c59903c5bac2868760e90fd521a4665aa7652088ac00e1f5050000000017a914\
-            3545e6e33b832c47050f24d3eeb93c9c03948bc787b32e1300000100fda50101000\
-            00000010289a3c71eab4d20e0371bbba4cc698fa295c9463afa2e397f8533ccb62f\
-            9567e50100000017160014be18d152a9b012039daf3da7de4f53349eecb985fffff\
-            fff86f8aa43a71dff1448893a530a7237ef6b4608bbb2dd2d0171e63aec6a4890b4\
-            0100000017160014fe3e9ef1a745e974d902c4355943abcb34bd5353ffffffff020\
-            0c2eb0b000000001976a91485cff1097fd9e008bb34af709c62197b38978a4888ac\
-            72fef84e2c00000017a914339725ba21efd62ac753a9bcd067d6c7a6a39d0587024\
-            7304402202712be22e0270f394f568311dc7ca9a68970b8025fdd3b240229f07f8a\
-            5f3a240220018b38d7dcd314e734c9276bd6fb40f673325bc4baa144c800d2f2f02\
-            db2765c012103d2e15674941bad4a996372cb87e1856d3652606d98562fe39c5e9e\
-            7e413f210502483045022100d12b852d85dcd961d2f5f4ab660654df6eedcc794c0\
-            c33ce5cc309ffb5fce58d022067338a8e0e1725c197fb1a88af59f51e44e4255b20\
-            167c8684031c05d1f2592a01210223b72beef0965d10be0778efecd61fcac6f79a4\
-            ea169393380734464f84f2ab300000000000000"
-        ).unwrap();
-
-        let psbt: PartiallySignedTransaction =
-            consensus::deserialize(&psbt_bytes).unwrap();
-
-        test_encoding_roundtrip(&psbt, &psbt_bytes).unwrap();
-    }
-
     #[test]
     fn test_encoding_extendedpubkey() {
         static EXT_PUBKEY1: [u8; 78] = [
