@@ -1,7 +1,7 @@
 // LNP/BP client-side-validation foundation libraries implementing LNPBP
 // specifications & standards (LNPBP-4, 7, 8, 9, 42, 81)
 //
-// Written in 2019-2021 by
+// Written in 2019-2022 by
 //     Dr. Maxim Orlovsky <orlovsky@pandoracore.com>
 //
 // To the extent possible under law, the author(s) have dedicated all
@@ -14,17 +14,19 @@
 
 use std::io;
 use std::io::{Read, Write};
+use std::sync::Arc;
 
 use bitcoin::consensus::ReadExt;
 use bitcoin::hashes::{hash160, ripemd160, sha256, sha256d};
+use bitcoin::XOnlyPublicKey;
 use miniscript::descriptor::{
-    Bare, DescriptorSinglePub, DescriptorXKey, InnerXKey, Pkh, Sh, ShInner,
-    Wildcard, Wpkh, Wsh, WshInner,
+    self, Descriptor, DescriptorPublicKey, DescriptorSinglePub, DescriptorXKey,
+    InnerXKey, SinglePubKey, TapTree, Wildcard,
 };
 use miniscript::policy::concrete::Policy;
 use miniscript::{
-    BareCtx, Descriptor, DescriptorPublicKey, Legacy, Miniscript,
-    MiniscriptKey, Segwitv0, Terminal,
+    BareCtx, Legacy, Miniscript, MiniscriptKey, ScriptContext, Segwitv0, Tap,
+    Terminal,
 };
 
 use crate::{Error, StrictDecode, StrictEncode};
@@ -47,6 +49,7 @@ const MS_KEY: u8 = 0x02;
 const MS_KEY_HASH: u8 = 0x03;
 const MS_THRESH: u8 = 0x20;
 const MS_MULTI: u8 = 0x21;
+const MS_MULTI_A: u8 = 0x2c;
 
 const MS_AFTER: u8 = 0x09;
 const MS_OLDER: u8 = 0x08;
@@ -98,6 +101,12 @@ macro_rules! strict_encode_usize {
     } };
 }
 
+impl From<miniscript::Error> for Error {
+    fn from(err: miniscript::Error) -> Self {
+        Error::DataIntegrityError(format!(": {}", err))
+    }
+}
+
 impl StrictEncode for BareCtx {
     #[inline]
     fn strict_encode<E: Write>(&self, _: E) -> Result<usize, Error> { Ok(0) }
@@ -131,6 +140,58 @@ impl StrictDecode for Segwitv0 {
     }
 }
 
+impl StrictEncode for Tap {
+    #[inline]
+    fn strict_encode<E: Write>(&self, _: E) -> Result<usize, Error> { Ok(0) }
+}
+
+impl StrictDecode for Tap {
+    fn strict_decode<D: Read>(_: D) -> Result<Self, Error> {
+        unreachable!("attempt to construct miniscript context object")
+    }
+}
+
+impl<Pk> StrictEncode for TapTree<Pk>
+where
+    Pk: MiniscriptKey + StrictEncode,
+    Pk::Hash: StrictEncode,
+{
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(match self {
+            TapTree::Tree(tree1, tree2) => {
+                let mut len = 2u8.strict_encode(&mut e)?;
+                len += tree1.strict_serialize()?.strict_encode(&mut e)?;
+                len += tree2.strict_serialize()?.strict_encode(&mut e)?;
+                len
+            }
+            TapTree::Leaf(pk) => {
+                strict_encode_list!(e; 1u8, pk)
+            }
+        })
+    }
+}
+
+impl<Pk> StrictDecode for TapTree<Pk>
+where
+    Pk: MiniscriptKey + StrictDecode,
+    Pk::Hash: StrictDecode,
+{
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        match u8::strict_decode(&mut d)? {
+            1u8 => Ok(TapTree::Leaf(StrictDecode::strict_decode(&mut d)?)),
+            2u8 => {
+                let vec1 = Vec::<u8>::strict_decode(&mut d)?;
+                let vec2 = Vec::<u8>::strict_decode(&mut d)?;
+                Ok(TapTree::Tree(
+                    Arc::new(TapTree::strict_deserialize(&vec1)?),
+                    Arc::new(TapTree::strict_deserialize(&vec2)?),
+                ))
+            }
+            wrong => Err(Error::EnumValueNotKnown("TapTree", wrong as usize)),
+        }
+    }
+}
+
 impl<Pk> StrictEncode for Policy<Pk>
 where
     Pk: MiniscriptKey + StrictEncode,
@@ -419,6 +480,16 @@ where
                     }
                     len
                 }
+                Terminal::MultiA(thresh, vec) => {
+                    let mut len = 1usize;
+                    e.write_all(&[MS_MULTI_A])?;
+                    len += strict_encode_usize!(e; *thresh);
+                    len += strict_encode_usize!(e; vec.len());
+                    for pk in vec {
+                        len += pk.strict_encode(&mut e)?;
+                    }
+                    len
+                }
             })
         }
 
@@ -534,6 +605,10 @@ where
                     }
                     Terminal::Thresh(thresh, vec)
                 }
+                MS_MULTI_A => Terminal::MultiA(
+                    d.read_u16()? as usize,
+                    Vec::strict_decode(d)?,
+                ),
 
                 wrong => {
                     return Err(Error::DataIntegrityError(format!(
@@ -558,14 +633,41 @@ where
 const DESCRIPTOR_BARE: u8 = 0x00;
 const DESCRIPTOR_PKH: u8 = 0x01;
 const DESCRIPTOR_SH: u8 = 0x02;
-const DESCRIPTOR_SH_SORTED_MULTI: u8 = 0x03;
-const DESCRIPTOR_SH_WPKH: u8 = 0x08;
-const DESCRIPTOR_SH_WSH: u8 = 0x09;
-const DESCRIPTOR_SH_WSH_SORTED_MULTI: u8 = 0x0a;
 const DESCRIPTOR_WPKH: u8 = 0x10;
 const DESCRIPTOR_WSH: u8 = 0x11;
-const DESCRIPTOR_WSH_SORTED_MULTI: u8 = 0x12;
-// Taproot: 0x2_
+const DESCRIPTOR_TR: u8 = 0x20;
+const DESCRIPTOR_SORTED_MULTI: u8 = 0x03;
+const DESCRIPTOR_MINISCRIPT: u8 = 0x04;
+
+impl StrictEncode for SinglePubKey {
+    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(match self {
+            SinglePubKey::FullKey(pk) => {
+                strict_encode_list!(e; 0x01u8, pk)
+            }
+            SinglePubKey::XOnly(xpk) => {
+                strict_encode_list!(e; 0x02u8, xpk)
+            }
+        })
+    }
+}
+
+impl StrictDecode for SinglePubKey {
+    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
+        Ok(match u8::strict_decode(&mut d)? {
+            0x01 => SinglePubKey::FullKey(bitcoin::PublicKey::strict_decode(
+                &mut d,
+            )?),
+            0x02 => SinglePubKey::XOnly(XOnlyPublicKey::strict_decode(&mut d)?),
+            wrong => {
+                return Err(Error::DataIntegrityError(format!(
+                    "unknown miniscript single pubkey tag `{:#04X}",
+                    wrong
+                )))
+            }
+        })
+    }
+}
 
 impl StrictEncode for DescriptorPublicKey {
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
@@ -668,37 +770,23 @@ where
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
         Ok(match self {
             Descriptor::Bare(bare) => {
-                strict_encode_list!(e; DESCRIPTOR_BARE, bare.as_inner())
+                strict_encode_list!(e; DESCRIPTOR_BARE, bare)
             }
             Descriptor::Pkh(pkh) => {
-                strict_encode_list!(e; DESCRIPTOR_PKH, pkh.as_inner())
+                strict_encode_list!(e; DESCRIPTOR_PKH, pkh)
             }
             Descriptor::Wpkh(wpkh) => {
-                strict_encode_list!(e; DESCRIPTOR_WPKH, wpkh.as_inner())
+                strict_encode_list!(e; DESCRIPTOR_WPKH, wpkh)
+            }
+            Descriptor::Sh(sh) => {
+                strict_encode_list!(e; DESCRIPTOR_SH, sh)
+            }
+            Descriptor::Wsh(wsh) => {
+                strict_encode_list!(e; DESCRIPTOR_WSH, wsh)
+            }
+            Descriptor::Tr(tr) => {
+                strict_encode_list!(e; DESCRIPTOR_TR, tr)
             }
-            Descriptor::Sh(sh) => match sh.as_inner() {
-                ShInner::Wsh(wsh) => match wsh.as_inner() {
-                    WshInner::SortedMulti(multi) => {
-                        strict_encode_list!(e; DESCRIPTOR_SH_WSH_SORTED_MULTI, multi.k, multi.pks)
-                    }
-                    WshInner::Ms(ms) => {
-                        strict_encode_list!(e; DESCRIPTOR_SH_WSH, ms)
-                    }
-                },
-                ShInner::Wpkh(wpkh) => {
-                    strict_encode_list!(e; DESCRIPTOR_SH_WPKH, wpkh.as_inner())
-                }
-                ShInner::SortedMulti(multi) => {
-                    strict_encode_list!(e; DESCRIPTOR_SH_SORTED_MULTI, multi.k, multi.pks)
-                }
-                ShInner::Ms(ms) => strict_encode_list!(e; DESCRIPTOR_SH, ms),
-            },
-            Descriptor::Wsh(wsh) => match wsh.as_inner() {
-                WshInner::SortedMulti(multi) => {
-                    strict_encode_list!(e; DESCRIPTOR_WSH_SORTED_MULTI, multi.k, multi.pks)
-                }
-                WshInner::Ms(ms) => strict_encode_list!(e; DESCRIPTOR_WSH, ms),
-            },
         })
     }
 }
@@ -709,49 +797,24 @@ where
     <Pk as MiniscriptKey>::Hash: StrictDecode,
 {
     fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        impl From<miniscript::Error> for Error {
-            fn from(err: miniscript::Error) -> Self {
-                Error::DataIntegrityError(format!(": {}", err))
-            }
-        }
-
         Ok(match u8::strict_decode(&mut d)? {
             DESCRIPTOR_BARE => {
-                Descriptor::Bare(Bare::new(Miniscript::strict_decode(&mut d)?)?)
+                Descriptor::Bare(descriptor::Bare::strict_decode(&mut d)?)
             }
             DESCRIPTOR_PKH => {
-                Descriptor::Pkh(Pkh::new(Pk::strict_decode(&mut d)?))
+                Descriptor::Pkh(descriptor::Pkh::strict_decode(&mut d)?)
             }
             DESCRIPTOR_SH => {
-                Descriptor::Sh(Sh::new(Miniscript::strict_decode(&mut d)?)?)
-            }
-            DESCRIPTOR_SH_SORTED_MULTI => Descriptor::Sh(Sh::new_sortedmulti(
-                usize::strict_decode(&mut d)?,
-                Vec::strict_decode(&mut d)?,
-            )?),
-            DESCRIPTOR_SH_WPKH => {
-                Descriptor::Sh(Sh::new_wpkh(Pk::strict_decode(&mut d)?)?)
-            }
-            DESCRIPTOR_SH_WSH => {
-                Descriptor::Sh(Sh::new_wsh(Miniscript::strict_decode(&mut d)?)?)
-            }
-            DESCRIPTOR_SH_WSH_SORTED_MULTI => {
-                Descriptor::Sh(Sh::new_wsh_sortedmulti(
-                    usize::strict_decode(&mut d)?,
-                    Vec::strict_decode(&mut d)?,
-                )?)
+                Descriptor::Sh(descriptor::Sh::strict_decode(&mut d)?)
             }
             DESCRIPTOR_WPKH => {
-                Descriptor::Wpkh(Wpkh::new(Pk::strict_decode(&mut d)?)?)
+                Descriptor::Wpkh(descriptor::Wpkh::strict_decode(&mut d)?)
             }
             DESCRIPTOR_WSH => {
-                Descriptor::Wsh(Wsh::new(Miniscript::strict_decode(&mut d)?)?)
+                Descriptor::Wsh(descriptor::Wsh::strict_decode(&mut d)?)
             }
-            DESCRIPTOR_WSH_SORTED_MULTI => {
-                Descriptor::Wsh(Wsh::new_sortedmulti(
-                    usize::strict_decode(&mut d)?,
-                    Vec::strict_decode(&mut d)?,
-                )?)
+            DESCRIPTOR_TR => {
+                Descriptor::Tr(descriptor::Tr::strict_decode(&mut d)?)
             }
             wrong => {
                 return Err(Error::DataIntegrityError(format!(
@@ -763,6 +826,273 @@ where
     }
 }
 
+impl<Pk> StrictEncode for descriptor::Bare<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Bare<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Self::new(StrictDecode::strict_decode(d)?).map_err(Error::from)
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::Pkh<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Pkh<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Ok(Self::new(StrictDecode::strict_decode(d)?))
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::Wpkh<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Wpkh<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Self::new(StrictDecode::strict_decode(d)?).map_err(Error::from)
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::Sh<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Sh<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        Ok(match descriptor::ShInner::strict_decode(d)? {
+            descriptor::ShInner::Wsh(wsh) => descriptor::Sh::new_with_wsh(wsh),
+            descriptor::ShInner::Wpkh(wpkh) => {
+                descriptor::Sh::new_with_wpkh(wpkh)
+            }
+            descriptor::ShInner::SortedMulti(inner) => {
+                descriptor::Sh::new_sortedmulti(inner.k, inner.pks)?
+            }
+            descriptor::ShInner::Ms(ms) => descriptor::Sh::new(ms)?,
+        })
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::Wsh<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    #[inline]
+    fn strict_encode<E: Write>(&self, e: E) -> Result<usize, Error> {
+        self.as_inner().strict_encode(e)
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Wsh<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    #[inline]
+    fn strict_decode<D: Read>(d: D) -> Result<Self, Error> {
+        match descriptor::WshInner::strict_decode(d)? {
+            descriptor::WshInner::SortedMulti(inner) => {
+                descriptor::Wsh::new_sortedmulti(inner.k, inner.pks)
+            }
+            descriptor::WshInner::Ms(ms) => descriptor::Wsh::new(ms),
+        }
+        .map_err(Error::from)
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::Tr<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.internal_key(), self.taptree()))
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::Tr<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        descriptor::Tr::new(
+            StrictDecode::strict_decode(&mut d)?,
+            StrictDecode::strict_decode(&mut d)?,
+        )
+        .map_err(Error::from)
+    }
+}
+
+impl<Pk, Ctx> StrictEncode for descriptor::SortedMultiVec<Pk, Ctx>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+    Ctx: ScriptContext,
+{
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(strict_encode_list!(e; self.k, self.pks))
+    }
+}
+
+impl<Pk, Ctx> StrictDecode for descriptor::SortedMultiVec<Pk, Ctx>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+    Ctx: ScriptContext,
+{
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        descriptor::SortedMultiVec::new(
+            StrictDecode::strict_decode(&mut d)?,
+            StrictDecode::strict_decode(&mut d)?,
+        )
+        .map_err(Error::from)
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::ShInner<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(match self {
+            descriptor::ShInner::Wsh(wsh) => {
+                strict_encode_list!(e; DESCRIPTOR_WSH, wsh)
+            }
+            descriptor::ShInner::Wpkh(wpkh) => {
+                strict_encode_list!(e; DESCRIPTOR_WPKH, wpkh)
+            }
+            descriptor::ShInner::SortedMulti(multi) => {
+                strict_encode_list!(e; DESCRIPTOR_SORTED_MULTI, multi)
+            }
+            descriptor::ShInner::Ms(ms) => {
+                strict_encode_list!(e; DESCRIPTOR_MINISCRIPT, ms)
+            }
+        })
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::ShInner<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        Ok(match u8::strict_decode(&mut d)? {
+            DESCRIPTOR_MINISCRIPT => {
+                descriptor::ShInner::Ms(StrictDecode::strict_decode(&mut d)?)
+            }
+            DESCRIPTOR_SORTED_MULTI => descriptor::ShInner::SortedMulti(
+                StrictDecode::strict_decode(&mut d)?,
+            ),
+            DESCRIPTOR_WPKH => {
+                descriptor::ShInner::Wpkh(StrictDecode::strict_decode(&mut d)?)
+            }
+            DESCRIPTOR_WSH => {
+                descriptor::ShInner::Wsh(StrictDecode::strict_decode(&mut d)?)
+            }
+            wrong => {
+                return Err(Error::DataIntegrityError(format!(
+                    "invalid miniscript ShInner descriptor type: #{:#04X}",
+                    wrong
+                )))
+            }
+        })
+    }
+}
+
+impl<Pk> StrictEncode for descriptor::WshInner<Pk>
+where
+    Pk: StrictEncode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictEncode,
+{
+    fn strict_encode<E: Write>(&self, mut e: E) -> Result<usize, Error> {
+        Ok(match self {
+            descriptor::WshInner::SortedMulti(multi) => {
+                strict_encode_list!(e; DESCRIPTOR_SORTED_MULTI, multi)
+            }
+            descriptor::WshInner::Ms(ms) => {
+                strict_encode_list!(e; DESCRIPTOR_MINISCRIPT, ms)
+            }
+        })
+    }
+}
+
+impl<Pk> StrictDecode for descriptor::WshInner<Pk>
+where
+    Pk: StrictDecode + MiniscriptKey,
+    <Pk as MiniscriptKey>::Hash: StrictDecode,
+{
+    fn strict_decode<D: Read>(mut d: D) -> Result<Self, Error> {
+        Ok(match u8::strict_decode(&mut d)? {
+            DESCRIPTOR_MINISCRIPT => {
+                descriptor::WshInner::Ms(StrictDecode::strict_decode(&mut d)?)
+            }
+            DESCRIPTOR_SORTED_MULTI => descriptor::WshInner::SortedMulti(
+                StrictDecode::strict_decode(&mut d)?,
+            ),
+            wrong => {
+                return Err(Error::DataIntegrityError(format!(
+                    "invalid miniscript WshInner descriptor type: #{:#04X}",
+                    wrong
+                )))
+            }
+        })
+    }
+}
+
 #[cfg(test)]
 mod test {
     use std::str::FromStr;
@@ -770,9 +1100,8 @@ mod test {
     use miniscript::{
         policy, BareCtx, Descriptor, Legacy, Miniscript, Segwitv0,
     };
+    use strict_encoding_test::*;
 
-    use crate::test_helpers::*;
-    // use crate::StrictEncode;
     use crate::StrictDecode;
 
     #[test]
@@ -812,7 +1141,7 @@ mod test {
 
     #[test]
     fn test_miniscript() {
-        const SET: [&str; 28] = [
+        const SET: [&str; 27] = [
             "lltvln:after(1231488000)",
             "uuj:and_v(v:multi(2,03d01115d548e7561b15c38f004d734633687cf4419620095bc5b0f47070afe85a,025601570cb47f238d2b0286db4a990fa0f3ba28d1a319f5e7cf55c2a2444da7cc),after(1231488000))",
             "or_b(un:multi(2,03daed4f2be3a8bf278e70132fb0beb7522f570e144bf615c07e996d443dee8729,024ce119c96e2fa357200b559b2f7dd5a5f02d5290aff74b03f3e471b273211c97),al:older(16))",
@@ -828,7 +1157,6 @@ mod test {
             "and_b(hash256(32ba476771d01e37807990ead8719f08af494723de1d228f2c2c07cc0aa40bac),a:and_b(hash256(131772552c01444cd81360818376a040b7c3b2b7b0a53550ee3edde216cec61b),a:older(1)))",
             "thresh(2,multi(2,03a0434d9e47f3c86235477c7b1ae6ae5d3442d49b1943c2b752a68e2a47e247c7,036d2b085e9e382ed10b69fc311a03f8641ccfff21574de0927513a49d9a688a00),a:multi(1,036d2b085e9e382ed10b69fc311a03f8641ccfff21574de0927513a49d9a688a00),ac:pk_k(022f01e5e15cca351daff3843fb70f3c2f0a1bdd05e5af888a67784ef3e10a2a01))",
             "and_n(sha256(d1ec675902ef1633427ca360b290b0b3045a0d9058ddb5e648b4c3c3224c5c68),t:or_i(v:older(4252898),v:older(144)))",
-            "or_d(d:and_v(v:older(4252898),v:older(4252898)),sha256(38df1c1f64a24a77b23393bca50dff872e31edc4f3b5aa3b90ad0b82f4f089b6))",
             "c:and_v(or_c(sha256(9267d3dbed802941483f1afa2a6bc68de5f653128aca9bf1461c5d0a3ad36ed2),v:multi(1,02c44d12c7065d812e8acf28d7cbb19f9011ecd9e9fdf281b0e6a3b5e87d22e7db)),pk_k(03acd484e2f0c7f65309ad178a9f559abde09796974c57e714c35f110dfc27ccbe))",
             "c:and_v(or_c(multi(2,036d2b085e9e382ed10b69fc311a03f8641ccfff21574de0927513a49d9a688a00,02352bbf4a4cdd12564f93fa332ce333301d9ad40271f8107181340aef25be59d5),v:ripemd160(1b0f3c404d12075c68c938f9f60ebea4f74941a0)),pk_k(03fff97bd5755eeea420453a14355235d382f6472f8568a18b2f057a1460297556))",
             "and_v(andor(hash256(8a35d9ca92a48eaade6f53a64985e9e2afeb74dcf8acb4c3721e0dc7e4294b25),v:hash256(939894f70e6c3a25da75da0cc2071b4076d9b006563cf635986ada2e93c0d735),v:older(50000)),after(499999999))",
@@ -874,7 +1202,7 @@ mod test {
 
         let secp = bitcoin::secp256k1::Secp256k1::new();
 
-        for s in &SET {
+        for s in SET {
             let (descr, _) = Descriptor::parse_descriptor(&secp, s).unwrap();
             test_object_encoding_roundtrip(&descr).unwrap();
         }
