@@ -1,7 +1,7 @@
 // LNP/BP client-side-validation foundation libraries implementing LNPBP
 // specifications & standards (LNPBP-4, 7, 8, 9, 42, 81)
 //
-// Written in 2019-2021 by
+// Written in 2019-2022 by
 //     Dr. Maxim Orlovsky <orlovsky@pandoracore.com>
 //
 // To the extent possible under law, the author(s) have dedicated all
@@ -34,12 +34,11 @@
 //! This list may be extended with future LNPBP-42 revisions
 
 use std::convert::TryFrom;
-use std::io;
 use std::net::{
     IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV6,
 };
 
-use crate::{strategies, Error, Strategy, StrictDecode, StrictEncode};
+use crate::{strategies, Strategy, StrictDecode, StrictEncode};
 
 /// Standard length of the host-specific part of the encoding, in bytes
 pub const ADDR_LEN: usize = 33; // Maximum Tor public key size
@@ -631,36 +630,6 @@ impl Uniform for SocketAddrV6 {
     }
 }
 
-impl StrictEncode for RawAddr {
-    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        e.write_all(self)?;
-        Ok(self.len())
-    }
-}
-
-impl StrictDecode for RawAddr {
-    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; ADDR_LEN];
-        d.read_exact(&mut ret)?;
-        Ok(ret)
-    }
-}
-
-impl StrictEncode for RawUniformAddr {
-    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        e.write_all(self)?;
-        Ok(self.len())
-    }
-}
-
-impl StrictDecode for RawUniformAddr {
-    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; UNIFORM_LEN];
-        d.read_exact(&mut ret)?;
-        Ok(ret)
-    }
-}
-
 impl Strategy for UniformAddr {
     type Strategy = strategies::UsingUniformAddr;
 }
