@@ -1,7 +1,7 @@
 // LNP/BP client-side-validation foundation libraries implementing LNPBP
 // specifications & standards (LNPBP-4, 7, 8, 9, 42, 81)
 //
-// Written in 2019-2021 by
+// Written in 2019-2022 by
 //     Dr. Maxim Orlovsky <orlovsky@pandoracore.com>
 //
 // To the extent possible under law, the author(s) have dedicated all
@@ -12,9 +12,9 @@
 // You should have received a copy of the Apache 2.0 License along with this
 // software. If not, see <https://opensource.org/licenses/Apache-2.0>.
 
+use std::borrow::Borrow;
 use std::cell::RefCell;
 use std::io;
-use std::ops::Deref;
 use std::rc::Rc;
 use std::sync::Arc;
 
@@ -31,63 +31,22 @@ impl StrictEncode for &[u8] {
     }
 }
 
-// TODO: #19 Re-implement with const generics once MSRV > 1.50
-
-impl StrictEncode for [u8; 16] {
+impl<const LEN: usize> StrictEncode for [u8; LEN] {
     fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
+        if LEN > u16::MAX as usize {
+            return Err(Error::ExceedMaxItems(LEN));
+        }
         e.write_all(self)?;
         Ok(self.len())
     }
 }
 
-impl StrictDecode for [u8; 16] {
+impl<const LEN: usize> StrictDecode for [u8; LEN] {
     fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; 16];
-        d.read_exact(&mut ret)?;
-        Ok(ret)
-    }
-}
-
-impl StrictEncode for [u8; 20] {
-    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        e.write_all(self)?;
-        Ok(self.len())
-    }
-}
-
-impl StrictDecode for [u8; 20] {
-    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; 20];
-        d.read_exact(&mut ret)?;
-        Ok(ret)
-    }
-}
-
-impl StrictEncode for [u8; 32] {
-    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        e.write_all(self)?;
-        Ok(self.len())
-    }
-}
-
-impl StrictDecode for [u8; 32] {
-    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; 32];
-        d.read_exact(&mut ret)?;
-        Ok(ret)
-    }
-}
-
-impl StrictEncode for [u8; 64] {
-    fn strict_encode<E: io::Write>(&self, mut e: E) -> Result<usize, Error> {
-        e.write_all(self)?;
-        Ok(self.len())
-    }
-}
-
-impl StrictDecode for [u8; 64] {
-    fn strict_decode<D: io::Read>(mut d: D) -> Result<Self, Error> {
-        let mut ret = [0u8; 64];
+        if LEN > u16::MAX as usize {
+            return Err(Error::ExceedMaxItems(LEN));
+        }
+        let mut ret = [0u8; LEN];
         d.read_exact(&mut ret)?;
         Ok(ret)
     }
@@ -95,7 +54,7 @@ impl StrictDecode for [u8; 64] {
 
 impl StrictEncode for Box<[u8]> {
     fn strict_encode<E: io::Write>(&self, e: E) -> Result<usize, Error> {
-        self.deref().strict_encode(e)
+        <[u8]>::borrow(self).strict_encode(e)
     }
 }
 
@@ -113,7 +72,7 @@ where
     T: StrictEncode,
 {
     fn strict_encode<E: io::Write>(&self, e: E) -> Result<usize, Error> {
-        self.deref().strict_encode(e)
+        T::borrow(self).strict_encode(e)
     }
 }
 
@@ -131,7 +90,7 @@ where
     T: StrictEncode,
 {
     fn strict_encode<E: io::Write>(&self, e: E) -> Result<usize, Error> {
-        self.deref().strict_encode(e)
+        self.borrow().strict_encode(e)
     }
 }
 
@@ -149,7 +108,7 @@ where
     T: StrictEncode,
 {
     fn strict_encode<E: io::Write>(&self, e: E) -> Result<usize, Error> {
-        self.deref().strict_encode(e)
+        T::borrow(self).strict_encode(e)
     }
 }
 
